/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.info.common;

import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.ObjectFactory;
import org.springframework.beans.factory.config.Scope;
import org.springframework.core.NamedInheritableThreadLocal;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Log4j2
public class CallScope implements Scope {
    public static final String SCOPE_NAME = "call";

    static final ThreadLocal<Map<String, Object>> scopedObjectsHolder = new NamedInheritableThreadLocal<>("Call Context");
    static final ThreadLocal<Map<String, Runnable>> destructionCallbacksHolder = new NamedInheritableThreadLocal<>(
            "Call Context Destruction Callbacks");

    public void startScope() {
        LOG.debug("START Call-Scope");
        scopedObjectsHolder.set(new ConcurrentHashMap<>());
        destructionCallbacksHolder.set(new ConcurrentHashMap<>());
    }

    public void endScope() {
        scopedObjectsHolder.remove();
        callAllDestructionCallbacks();
        destructionCallbacksHolder.remove();
        LOG.debug("END Call-Scope");
    }

    void callAllDestructionCallbacks() {
        destructionCallbacksHolder.get().values().forEach(Runnable::run);
    }

    @Override
    public @NonNull Object get(@NonNull String name, @NonNull ObjectFactory<?> objectFactory) {
        if (!scopedObjectsHolder.get().containsKey(name)) {
            scopedObjectsHolder.get().put(name, objectFactory.getObject());
        }

        return scopedObjectsHolder.get().get(name);
    }

    @Override
    public Object remove(@NonNull String name) {
        destructionCallbacksHolder.get().remove(name);
        return scopedObjectsHolder.get().remove(name);
    }

    @Override
    public void registerDestructionCallback(@NonNull String name, @NonNull Runnable callback) {
        destructionCallbacksHolder.get().put(name, callback);

    }

    @Override
    public Object resolveContextualObject(@NonNull String key) {
        return scopedObjectsHolder.get().get(key);
    }

    @Override
    public String getConversationId() {
        return Thread.currentThread().getName();
    }
}
