/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.info.common;

import io.jsonwebtoken.*;
import io.jsonwebtoken.security.Keys;
import io.jsonwebtoken.security.SignatureException;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Optional;

@Log4j2
@Component
public class JwtTokenVerifier {

    @Value("${ozgcloud.jwt.secret}")
    private String jwtSecret;

    public Optional<JwtParser> validate(String token) {
        try {
            byte[] signingKey = jwtSecret.getBytes();

            var parser = Jwts.parser()
                    .verifyWith(Keys.hmacShaKeyFor(signingKey))
                    .build();

            return Optional.of(parser);
        } catch (ExpiredJwtException exception) {
            LOG.error("Token {} expired. {}", token, exception.getMessage());
        } catch (MalformedJwtException exception) {
            LOG.error("Token {} malformed. {}", token, exception.getMessage());
        } catch (SignatureException exception) {
            LOG.error("Token {} signature is invalid. {}", token, exception.getMessage());
        } catch (IllegalArgumentException | UnsupportedJwtException exception) {
            LOG.error("Error parsing token {}. Cause: {}", token, exception.getMessage());
        }
        return Optional.empty();
    }
}
