/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.info.infomanager;

import de.ozgcloud.info.GrpcInformationRequest;
import de.ozgcloud.info.GrpcInformationResponse;
import de.ozgcloud.info.InformationServiceGrpc;
import de.ozgcloud.info.common.ContextService;
import de.ozgcloud.info.common.JwtTokenVerifier;
import de.ozgcloud.info.common.User;
import de.ozgcloud.info.nachricht.GrpcNachricht;
import io.grpc.stub.StreamObserver;
import io.jsonwebtoken.Jwt;
import io.jsonwebtoken.impl.DefaultClaims;
import java.util.ArrayList;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;
import net.devh.boot.grpc.server.service.GrpcService;
import org.springframework.util.ObjectUtils;

@Log4j2
@GrpcService
@RequiredArgsConstructor
public class InformationGrpcService extends InformationServiceGrpc.InformationServiceImplBase {
    private final InformationService service;
    private final ContextService contextService;
    private final InfoManagerProperties infoManagerProperties;
    private final JwtTokenVerifier tokenVerifier;

    @Override
    public void getInformation(GrpcInformationRequest request,
                               StreamObserver<GrpcInformationResponse> responseStreamObserver) {
        if (checkForAccessToPostfach(request.getPostfachId(), responseStreamObserver, tokenVerifier)) {
            var nachrichten = new ArrayList<GrpcNachricht>();
            var information = service.getInformationOfPostfach(request.getPostfachId());

            information.forEach(info -> nachrichten.add(GrpcNachricht.newBuilder()
                .setPostfachId(info.getPostfachId())
                .setNachrichtenListUrl(info.getNachrichtenListUrl())
                .build()
            ));

            responseStreamObserver.onNext(GrpcInformationResponse.newBuilder().addAllNachrichten(nachrichten).build());
            responseStreamObserver.onCompleted();
        }
    }

    boolean checkForAccessToPostfach(String requestedPostfachId,
                                     StreamObserver<GrpcInformationResponse> responseStreamObserver,
                                     JwtTokenVerifier jwtTokenVerifier) {
        boolean success = false;

        try {
            var token = contextService.getRequestAttributes().getJwtToken();

            var jwtClaims =
                jwtTokenVerifier.validate(token).map(parser -> parser.parse(token)).map(Jwt::getPayload).stream()
                    .toList();
            User user = User.fromClaims((DefaultClaims) jwtClaims.get(0));

            if (ObjectUtils.isEmpty(user.getPostkorbHandle())) {
                var msg = "Invalid postfach_id_key_name [%s] configured. Unable get Postfach id from token".formatted(
                    infoManagerProperties.getPostfachIdKeyName());
                LOG.error(msg);
                throw new IllegalStateException(msg);
            }

            if (!user.getPostkorbHandle().equals(requestedPostfachId)) {
                LOG.error("User " + user.getUsername() + " is not allowed to access events of postfach id " +
                    requestedPostfachId);
                throw new SecurityException("User not allowed");
            }

            success = true;
        } catch (Exception e) {
            LOG.error("Error verifying jwt token. " + e.getMessage());
            responseStreamObserver.onError(e);
        }


        return success;
    }
}
