/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.info.infomanager;

import static de.ozgcloud.info.NachrichtEventTestFactory.EMPTY_POSTFACH_HANDLE;
import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.any;
import static org.mockito.Mockito.anyString;
import static org.mockito.Mockito.atMostOnce;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import de.ozgcloud.info.GrpcInformationRequest;
import de.ozgcloud.info.GrpcInformationResponse;
import de.ozgcloud.info.JwtTokenProvider;
import de.ozgcloud.info.NachrichtEventTestFactory;
import de.ozgcloud.info.common.ContextService;
import de.ozgcloud.info.common.JwtTokenVerifier;
import de.ozgcloud.info.common.RequestAttributes;
import de.ozgcloud.info.common.UserTestFactory;
import de.ozgcloud.info.nachricht.NachrichtEvent;
import io.grpc.stub.StreamObserver;
import io.jsonwebtoken.Jwt;
import io.jsonwebtoken.JwtParser;
import io.jsonwebtoken.impl.DefaultClaims;
import io.jsonwebtoken.impl.DefaultJwt;
import io.jsonwebtoken.impl.DefaultJwtHeaderBuilder;
import java.util.List;
import java.util.Optional;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

@ExtendWith(MockitoExtension.class)
class InformationGrpcServiceTest {

    private static final String POSTFACH_ID_KEY = "postkorbhandle";
    private static final String POSTFACH_ID = "other";
    private static final String TRUST_LEVEL_KEY = "trustlevel";
    private static final String TRUST_LEVEL = "basic";

    @Spy
    @InjectMocks
    InformationGrpcService informationGrpcService;

    @Mock
    InformationService service;

    @Spy
    StreamObserver<GrpcInformationResponse> informationResponseStreamObserver;

    @Captor
    ArgumentCaptor<GrpcInformationResponse> informationResponseArgumentCaptor;

    @Mock
    private RequestAttributes requestAttributes;

    @Mock
    private ContextService contextService;

    @Mock
    private JwtTokenVerifier tokenVerifier;

    @Nested
    class TestInformationAvailable {
        final NachrichtEvent firstNachrichtEvent = NachrichtEventTestFactory.create();

        @BeforeEach
        void init() {
            var information = List.of(firstNachrichtEvent, NachrichtEventTestFactory.create());
            when(service.getInformationOfPostfach(anyString())).thenReturn(information);
            when(requestAttributes.getJwtToken()).thenReturn(JwtTokenProvider.generate(UserTestFactory.create()));
            when(contextService.getRequestAttributes()).thenReturn(requestAttributes);
            initJwt();
        }


        @Test
        void shouldCallService() {
            informationGrpcService.getInformation(
                GrpcInformationRequest.newBuilder().setPostfachId(NachrichtEventTestFactory.POSTKORB_HANDLE).build(),
                informationResponseStreamObserver
            );

            verify(service, atMostOnce()).getInformationOfPostfach(anyString());
        }

        @Test
        void shouldCreateInformation() {
            informationGrpcService.getInformation(
                GrpcInformationRequest.newBuilder().setPostfachId(NachrichtEventTestFactory.POSTKORB_HANDLE).build(),
                informationResponseStreamObserver
            );

            verify(informationResponseStreamObserver, atMostOnce()).onNext(informationResponseArgumentCaptor.capture());

            var response = informationResponseArgumentCaptor.getValue();
            assertThat(response.getNachrichtenList()).hasSize(2);
            var information = response.getNachrichtenList().get(0);
            assertThat(information.getPostfachId()).isEqualTo(firstNachrichtEvent.getPostfachId());
            assertThat(information.getNachrichtenListUrl()).isEqualTo(firstNachrichtEvent.getNachrichtenListUrl());
        }

        @Test
        void shouldSetResponse() {
            informationGrpcService.getInformation(
                GrpcInformationRequest.newBuilder().setPostfachId(NachrichtEventTestFactory.POSTKORB_HANDLE).build(),
                informationResponseStreamObserver
            );

            verify(informationResponseStreamObserver, atMostOnce()).onNext(any(GrpcInformationResponse.class));
        }
    }

    @Nested
    class TestNoInformationAvailable {

        @Test
        void responseShouldHaveEmptyList() {
            initJwt();
            when(service.getInformationOfPostfach(anyString())).thenReturn(List.of());
            when(requestAttributes.getJwtToken()).thenReturn(JwtTokenProvider.generate(UserTestFactory.create()));
            when(contextService.getRequestAttributes()).thenReturn(requestAttributes);

            informationGrpcService.getInformation(
                GrpcInformationRequest.newBuilder().setPostfachId(NachrichtEventTestFactory.POSTKORB_HANDLE).build(),
                informationResponseStreamObserver
            );

            verify(informationResponseStreamObserver, atMostOnce()).onNext(informationResponseArgumentCaptor.capture());

            var response = informationResponseArgumentCaptor.getValue();
            assertThat(response.getNachrichtenList()).isEmpty();
        }

        @Test
        void shouldSetResponse() {
            informationGrpcService.getInformation(
                GrpcInformationRequest.newBuilder().setPostfachId(NachrichtEventTestFactory.POSTKORB_HANDLE).build(),
                informationResponseStreamObserver
            );

            verify(informationResponseStreamObserver, atMostOnce()).onNext(any(GrpcInformationResponse.class));
        }
    }

    @Nested
    class TestAccessCheck {
        @Mock
        StreamObserver<GrpcInformationResponse> responseStreamObserver;

        @BeforeEach
        void init() {

        }

        @Test
        void shouldAccessPostfach() {
            initJwt();
            when(requestAttributes.getJwtToken()).thenReturn(JwtTokenProvider.generate(UserTestFactory.create()));
            when(contextService.getRequestAttributes()).thenReturn(requestAttributes);

            var res = informationGrpcService.checkForAccessToPostfach(NachrichtEventTestFactory.POSTKORB_HANDLE,
                responseStreamObserver, tokenVerifier);

            assertThat(res).isTrue();
        }

        @Test
        void shouldNotAccessPostfach() {
            var res = informationGrpcService.checkForAccessToPostfach(EMPTY_POSTFACH_HANDLE, responseStreamObserver,
                tokenVerifier);

            assertThat(res).isFalse();
        }

        @Test
        void shouldThrowSecurityException() {
            initJwt();
            when(requestAttributes.getJwtToken()).thenReturn(JwtTokenProvider.generate(UserTestFactory.create()));
            when(contextService.getRequestAttributes()).thenReturn(requestAttributes);

            informationGrpcService.checkForAccessToPostfach(EMPTY_POSTFACH_HANDLE, responseStreamObserver,
                tokenVerifier);

            verify(responseStreamObserver).onError(any(SecurityException.class));
        }
    }

    @Nested
    class TestExceptionOnOtherPostfachId {
        @Mock
        StreamObserver<GrpcInformationResponse> responseStreamObserver;

        @Mock
        private JwtTokenVerifier tokenVerifier;

        @BeforeEach
        void init() {
            when(requestAttributes.getJwtToken()).thenReturn(JwtTokenProvider.generate(UserTestFactory.create()));
            when(contextService.getRequestAttributes()).thenReturn(requestAttributes);

            JwtParser jwtParser = mock(JwtParser.class);
            var jwtHeader = new DefaultJwtHeaderBuilder().build();
            var jwtPayload = mock(DefaultClaims.class);
            when(jwtPayload.get(POSTFACH_ID_KEY, String.class)).thenReturn(POSTFACH_ID);
            when(jwtPayload.get(TRUST_LEVEL_KEY, String.class)).thenReturn(TRUST_LEVEL);
            when(jwtParser.parse(anyString())).thenReturn((Jwt) new DefaultJwt<>(jwtHeader, jwtPayload));
            var jwtParserOptional = Optional.of(jwtParser);
            when(tokenVerifier.validate(any())).thenReturn(jwtParserOptional);
        }

        @Test
        void shouldThrowSecurityException() {
            informationGrpcService.checkForAccessToPostfach(EMPTY_POSTFACH_HANDLE, responseStreamObserver,
                tokenVerifier);

            verify(responseStreamObserver).onError(any(SecurityException.class));
        }
    }

    @Test
    void shouldCallOnCompleteForGrpcInformationRequest() {
        informationGrpcService.getInformation(
            GrpcInformationRequest.newBuilder().setPostfachId(NachrichtEventTestFactory.POSTKORB_HANDLE).build(),
            informationResponseStreamObserver
        );

        verify(informationResponseStreamObserver, atMostOnce()).onCompleted();
    }

    private void initJwt() {
        JwtParser jwtParser = mock(JwtParser.class);
        var jwtHeader = new DefaultJwtHeaderBuilder().build();
        var jwtPayload = mock(DefaultClaims.class);
        when(jwtPayload.get(POSTFACH_ID_KEY, String.class)).thenReturn(UserTestFactory.POSTKORB_HANDLE);
        when(jwtPayload.get(TRUST_LEVEL_KEY, String.class)).thenReturn(TRUST_LEVEL);
        when(jwtParser.parse(anyString())).thenReturn((Jwt) new DefaultJwt<>(jwtHeader, jwtPayload));
        var jwtParserOptional = Optional.of(jwtParser);
        when(tokenVerifier.validate(any())).thenReturn(jwtParserOptional);
    }
}