/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.info.nachricht;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.any;
import static org.mockito.Mockito.atMostOnce;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.verify;

import de.ozgcloud.info.NachrichtEventTestFactory;
import io.grpc.stub.StreamObserver;
import jakarta.validation.ConstraintViolationException;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

@ExtendWith(MockitoExtension.class)
class NachrichtEventGrpcServiceTest {

    @Spy
    @InjectMocks
    NachrichtenGrpcService nachrichtenGrpcService;

    @Mock
    NachrichtenService nachrichtenService;

    @Nested
    class TestSaveNewNachrichtEvent {
        @Mock
        StreamObserver<GrpcNewNachrichtReply> newNachrichtStreamObserver;
        NachrichtEvent nachrichtEvent = NachrichtEventTestFactory.create();

        GrpcNewNachrichtRequest newNachrichtRequest = GrpcNewNachrichtRequest.newBuilder().setNachricht(
            GrpcNachricht.newBuilder()
                .setPostfachId(nachrichtEvent.getPostfachId())
                .setNachrichtenListUrl(nachrichtEvent.getNachrichtenListUrl())
        ).build();

        @Captor
        ArgumentCaptor<NachrichtEvent> nachrichtArgumentCaptor;

        @Test
        void shouldCallService() {
            nachrichtenGrpcService.saveNewNachricht(newNachrichtRequest, newNachrichtStreamObserver);

            verify(nachrichtenService, atMostOnce()).save(any(NachrichtEvent.class));
        }

        @Test
        void shouldCreateNachricht() {
            nachrichtenGrpcService.saveNewNachricht(newNachrichtRequest, newNachrichtStreamObserver);

            verify(nachrichtenService, atMostOnce()).save(nachrichtArgumentCaptor.capture());

            var savedNachricht = nachrichtArgumentCaptor.getValue();
            assertThat(savedNachricht.getPostfachId()).isEqualTo(nachrichtEvent.getPostfachId());
            assertThat(savedNachricht.getNachrichtenListUrl()).isEqualTo(nachrichtEvent.getNachrichtenListUrl());
        }

        @Test
        void shouldCallOnNext() {
            nachrichtenGrpcService.saveNewNachricht(newNachrichtRequest, newNachrichtStreamObserver);

            verify(newNachrichtStreamObserver, atMostOnce()).onNext(any(GrpcNewNachrichtReply.class));
        }

        @Test
        void shouldCallOnCompleted() {
            nachrichtenGrpcService.saveNewNachricht(newNachrichtRequest, newNachrichtStreamObserver);

            verify(newNachrichtStreamObserver, atMostOnce()).onCompleted();
        }

        @Test
        void shouldCallOnError() {
            doThrow(ConstraintViolationException.class).when(nachrichtenService).save(any(NachrichtEvent.class));

            nachrichtenGrpcService.saveNewNachricht(newNachrichtRequest, newNachrichtStreamObserver);

            verify(newNachrichtStreamObserver, atMostOnce()).onError(any());
        }
    }

    @Nested
    class TestFinishNachrichtEvent {
        @Mock
        StreamObserver<GrpcFinishedNachrichtReply> finishedNachrichtReplyStreamObserver;

        GrpcFinishedNachrichtRequest finishedNachrichtRequest = GrpcFinishedNachrichtRequest.newBuilder().build();

        @Test
        void shouldCallOnNext() {
            nachrichtenGrpcService.finishNachricht(finishedNachrichtRequest, finishedNachrichtReplyStreamObserver);

            verify(finishedNachrichtReplyStreamObserver, atMostOnce()).onNext(any(GrpcFinishedNachrichtReply.class));
        }

        @Test
        void shouldCallOnCompleted() {
            nachrichtenGrpcService.finishNachricht(finishedNachrichtRequest, finishedNachrichtReplyStreamObserver);

            verify(finishedNachrichtReplyStreamObserver, atMostOnce()).onCompleted();
        }

    }
}