/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.info.nachricht;

import static de.ozgcloud.info.NachrichtEventTestFactory.POSTKORB_HANDLE;
import static de.ozgcloud.info.NachrichtEventTestFactory.URL;
import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatExceptionOfType;
import static org.mockito.Mockito.any;
import static org.mockito.Mockito.atMostOnce;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.never;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

import de.ozgcloud.info.NachrichtEventTestFactory;
import jakarta.validation.ConstraintViolationException;
import java.util.Optional;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.ArgumentCaptor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;

@ExtendWith(MockitoExtension.class)
class NachrichtEventServiceTest {

    @Spy
    @InjectMocks
    private NachrichtenService service;

    @Mock
    private NachrichtenRepository repository;

    private final ArgumentCaptor<NachrichtEvent> argument = ArgumentCaptor.forClass(NachrichtEvent.class);

    @Nested
    class TestAddingNewNachrichtEvent {

        @Test
        void shouldHavePostfachIdAndNachrichtenListUrl() {
            service.save(NachrichtEventTestFactory.create());

            verify(repository, atMostOnce()).findByPostfachIdAndNachrichtenListUrl(POSTKORB_HANDLE, URL);
            verify(repository, atMostOnce()).save(argument.capture());

            assertThat(argument.getValue().getPostfachId()).isEqualTo(POSTKORB_HANDLE);
            assertThat(argument.getValue().getNachrichtenListUrl()).isEqualTo(URL);
        }

        @Test
        void shouldHandleConstraintValidationException() {
            doThrow(ConstraintViolationException.class).when(repository).save(any(NachrichtEvent.class));

            var nachrichtEvent = NachrichtEventTestFactory.create();
            assertThatExceptionOfType(ConstraintViolationException.class).isThrownBy(
                () -> service.save(nachrichtEvent));
        }
    }

    @Nested
    class TestExistingNachrichtEvent {

        @Test
        void shouldHandleRedundantNachrichten() {
            NachrichtEvent nachrichtEvent = NachrichtEventTestFactory.create();

            when(repository.findByPostfachIdAndNachrichtenListUrl(POSTKORB_HANDLE, URL)).thenReturn(
                Optional.of(nachrichtEvent));

            service.save(nachrichtEvent);

            verify(repository, atMostOnce()).findByPostfachIdAndNachrichtenListUrl(POSTKORB_HANDLE, URL);
            verify(repository, never()).save(argument.capture());
        }
    }
}
